--[[
Name:		  LibFrameX-1.0
Author(s): 	  Azmaedus
Revision: 	  $Rev: 1 $
Created:	  01/23/2015
Description:  Library that creates a frame structure dynamically.
Dependencies: LibStub
License:	  GPLv2

	Change Log
	--------------------
	01/23/2015 - 1.0a 		Creation of addon
	01/24/2015 - 1.0a-r2	Changed the name from GJBPlayer-1.0 to LibFrameX-1.0
	01/24/2015 - 1.0a-r3	Optimization pass
]]

local LIBFRAMEX = "LibFrameX-1.0"
local LIBFRAMEX_MINOR = 0 + tonumber(("$Revision: 3 $"):match("(%d+)"))

local FrameX = LibStub:NewLibrary(LIBFRAMEX, LIBFRAMEX_MINOR)
if not FrameX then return end

-- FrameX publics
FrameX.tframes = {}

-- Default settings for LFX frame
local defaults = {
	-- Background
	BG = {
		-- Backdrop info
		BD = {
			bgFile = [[Interface\ChatFrame\ChatFrameBackground]],
			edgeFile = [[Interface/Tooltips/UI-Tooltip-Border]],
			tile = true, tileSize = 4, edgeSize = 4,
			insets = { left = 1, right = 1, top = 1, bottom = 1 }
		},
		align = "CENTER", xPos = 0, yPos = 0, Scale = 1, w = 200,	h = 150, r = 0,	g = 0, b = 0, a = 1, br = 1, bg = 1, bb = 1, ba = 1,
	},
}

if not FrameX.frame then
	FrameX.frame = CreateFrame("Frame")
	local f, BG = FrameX.frame, defaults.BG
	f:SetWidth(BG.w)
	f:SetHeight(BG.h)
	f:SetBackdrop(BG.BD)
	f:SetBackdropColor(BG.r, BG.g, BG.b, BG.a)
	f:SetBackdropBorderColor(BG.br, BG.bg, BG.bb, BG.ba)
	f.pos = {}
	f:Show()
end

-- Localization enUS
local L_FRAMEX_TITLE = "LibFrameX-1.0 Frame"
local L_FRAMEX_ERROR = "Error creating frame!"

local l = GetLocale()
if l == "frFR" then
	L_FRAMEX_TITLE = "LibFrameX-1.0 Frame"
	L_FRAMEX_ERROR = "Erreur lors de la creation de la frame!"
elseif l == "koKR" then
	-- Missing
elseif l == "deDE" then
	-- Missing
elseif l == "zhCN" then
	-- Missing
elseif l == "zhTW" then
	-- Missing
elseif l == "ruRU" then
	-- Missing
elseif l == "esES" or l == "esMX" then
	-- Missing
elseif l == "ptBR" then
	-- Missing
elseif l == "itIT" then
	-- Missing
end

-- Returns the index of a frame in LFX.tframes by its name
function FrameX:findFrame(name)
	for k, v in ipairs(self.tframes) do
		if v.name == name then
			return k
		end
	end
	return nil
end

-- Function to create a sub-frame
function FrameX:genFrame(f, parent)
	if not f then return end -- fail
	
	local fo = {}
	-- TODO: gotta be a better way to do this
	if parent == UIParent then
		fo = CreateFrame(f.ftype, f.name, UIParent)
	else
		fo = CreateFrame(f.ftype, f.name, parent, f.template)
	end
	fo.name = f.name
	fo.parentrgba = f.parentrgba
	fo.parentdim = f.parentdim
	fo.parentname = f.parentname
	fo.childrgba = f.childrgba
	fo.childrgbashaded = f.childrgbashaded
	fo.childbrgba = f.childbrgba
	fo.childbd = f.childbd
	fo.childdim = f.childdim
	fo.rgba = f.rgba
	fo.rgbashaded = f.rgbashaded
	fo.brgba = f.brgba
	fo.bd = f.bd
	fo.dim = f.dim
	fo.root = f.root
	
	if f.parentrgba and f.parentbrgba and f.parentbd then
		local rgba = {}
		local brgba = {}
		local bd = {}
		if parent and not f.root then
			local id = self:findFrame(f.parentname)
			if id then
				pf = self.tframes[id]
				rgba = pf.childrgba
				brgba = pf.childbrgba
				bd = pf.childbd
			else
				rgba = f.rgba
				brgba = f.brgba
				bd = f.bd
			end
		else
			rgba = f.rgba
			brgba = f.brgba
			bd = f.bd
		end
		fo:SetBackdrop(bd)
		fo:SetBackdropColor(rgba.r/255, rgba.g/255, rgba.b/255, rgba.a)
		fo:SetBackdropBorderColor(brgba.r/255, brgba.g/255, brgba.b/255, brgba.a)
	elseif f.bd then
		fo:SetBackdrop(f.bd)
		fo:SetBackdropColor(f.rgba.r/255, f.rgba.g/255, f.rgba.b/255, f.rgba.a)
		fo:SetBackdropBorderColor(f.brgba.r/255, f.brgba.g/255, f.brgba.b/255, f.brgba.a)
	end

	if f.parentdim then
		local dim = {}
		if parent then
			local id = self:findFrame(f.parentname)
			if id then
				dim = self.tframes[id].childdim
			else
				dim = f.dim
			end
		else
			dim = f.dim
		end
		fo:SetWidth(dim.w)
		fo:SetHeight(dim.h)
	else
		fo:SetWidth(f.dim.w)
		fo:SetHeight(f.dim.h)
	end

	if f.bd then
	end
	
	if f.show then
		fo:Show()
	end
	return fo
end

--[[
Creates all sub frames by passing a table of frames to create
Structure of the tframes table:
	t = {
		{
			root = true, -- This frame's graphics wont be drawn; acts as a container
			ftype = "type", --Frame, Button, ...
			name = "FrameName",
			rgba = {r = 1, g = 1, b = 1, a = 1}, -- red, green, blue, alpha 0.0 to 1.0
			dim = {w = 1, h = 1, s = 1}, -- width, height, scale
			pos = {x = 0, y = 0, a = "ANCHOR", r = "PARENT", ap = "PARENT-ANCHOR"}, -- x, y, anchor, relative to, r anchor
			bd = {}, -- BackDrop
			brgba = {r = 1, g = 1, b = 1, a = 1}, -- BackDrop : red, green, blue, alpha 0.0 to 1.0
			show = true,
			movable = {
				enabled = false, 
				drag = {
					"LeftButton",
					...
				}
			},
			clicks = {"LeftButton","RightButton","Button4"},
			scripts = {
				{event = "OnSomething", func = function(self,...)end},
				...
			},
		},
		...
	}
]]
FrameX.prev = UIParent

function FrameX:makeFrames(t)
FrameX.tframes = {}
	for k, v in ipairs(t) do
		-- check if frame exists
		if v then
			self.tframes[k] = self:genFrame(v, self.prev) -- create the frame
			xf = self.tframes[k]
			self.prev = xf -- set previous frame object
			xf:SetPoint(v.pos.a, v.pos.r, v.pos.rp, v.pos.x, v.pos.y) -- align, relative, rel parent, x, y
			
			-- check if the frame has a text component
			if v.text then
				xf.text = xf:CreateFontString(nil, "ARTWORK", v.text.fs or "NumberFont_Shadow_Small")
				xf.text:SetAllPoints(true)
				xf.text:SetJustifyH(v.text.halign or "CENTER")
				xf.text:SetJustifyV(v.valign or "CENTER")
				xf.text:SetTextColor(v.text.color.r/255 or 1, v.text.color.g/255 or 1, v.text.color.b/255 or 1, v.text.color.a or 1)
				xf.text:SetText(v.text.str or "")
			end

			--TODO: set texture
			--FrameX.tframes[i]:SetNormalTexture("\\media\\button.tga")
			
			-- if frame is movable
			if v.movable and v.movable.enabled then
				xf:SetMovable(true)
				xf:EnableMouse(true)
				xf:RegisterForDrag(unpack(v.movable.drag))
				--xf.text:SetScale(v.dim.s) ????? why di I put this here
			end

			-- register click events
			if v.clicks then
				FrameX.tframes[i]:SetPoint("CENTER")
				FrameX.tframes[i]:RegisterForClicks(unpack(v.clicks))
			end
			
			-- create scripts
			if v.scripts then -- be careful, if you have scripts, you should have a register clicks
				for ky, vy in ipairs(v.scripts) do
					xf:SetScript(vy.event, vy.func)
				end
			end
		else
			assert(false, L_FRAMEX_ERROR .. " #" .. k) -- Error
		end
	end
end

function FrameX:setVisible(visible)
	if visible then
		for _, v in ipairs(FrameX.tframes) do
			v:Show()
		end
	else
		for _, v in ipairs(FrameX.tframes) do
			v:Hide()
		end
	end
end

function FrameX:setFrameVisible(i, state)
	if state then
		self.tframes[i]:Show()
	else
		self.tframes[i]:Hide()
	end
end

function FrameX:setPosition(a, x, y)
	FrameX.tframes[1]:SetPoint(a, p, nil, x, y)
end
